@@ -4,8 +4,11 @@ Info action
 
 import collections
 import csv
+import multiprocessing
 import sys
 
+from functools import partial
+
 from Bio import SeqIO
 
 from seqmagick import fileformat
@@ -17,7 +20,7 @@ def build_parser(parser):
     parser.add_argument('--input-format', help="""Input format. Overrides
             extension for all input files""")
     parser.add_argument('--out-file', dest='destination_file',
-            type=common.FileType('w'), default=sys.stdout,
+            type=common.FileType('wt'), default=sys.stdout,
             metavar='destination_file',
             help='Output destination. Default: STDOUT')
     parser.add_argument('--format', dest='output_format',
@@ -25,16 +28,19 @@ def build_parser(parser):
         tab-delimited, CSV or aligned in a borderless table.  Default is
         tab-delimited if the output is directed to a file, aligned if output to
         the console.""")
+    parser.add_argument('--threads', default=1,
+            type=int,
+            help="""Number of threads (CPUs). [%(default)s] """)
 
 class SeqInfoWriter(object):
     """
     Base writer for sequence files
     """
 
-    def __init__(self, sequence_files, output, input_format=None):
+    def __init__(self, sequence_files, rows, output):
         self.sequence_files = sequence_files
+        self.rows = rows
         self.output = output
-        self.input_format = input_format
 
     def write_row(self, row):
         raise NotImplementedError("Override in subclass")
@@ -45,18 +51,16 @@ class SeqInfoWriter(object):
     def write(self):
         header = ('name', 'alignment', 'min_len', 'max_len', 'avg_len',
                   'num_seqs')
-        self.write_header(header)
-        rows = (summarize_sequence_file(source_file, self.input_format)
-                for source_file in self.sequence_files)
 
-        for row in rows:
-            self.write_row(row)
+        self.write_header(header)
+
+        for row in self.rows:
+            self.write_row(_SeqFileInfo(*row))
 
 class CsvSeqInfoWriter(SeqInfoWriter):
     delimiter = ','
-    def __init__(self, sequence_files, output, input_format=None):
-        super(CsvSeqInfoWriter, self).__init__(sequence_files, output,
-                input_format)
+    def __init__(self, sequence_files, rows, output):
+        super(CsvSeqInfoWriter, self).__init__(sequence_files, rows, output)
         self.writer = csv.writer(self.output, delimiter=self.delimiter,
                 lineterminator='\n')
 
@@ -70,21 +74,19 @@ class TsvSeqInfoWriter(CsvSeqInfoWriter):
     delimiter = '\t'
 
 class AlignedSeqInfoWriter(SeqInfoWriter):
-    def __init__(self, sequence_files, output, input_format=None):
-        super(AlignedSeqInfoWriter, self).__init__(sequence_files, output,
-                input_format)
+    def __init__(self, sequence_files, rows, output):
+        super(AlignedSeqInfoWriter, self).__init__(sequence_files, rows, output)
         self.max_name_length = max(len(f) for f in self.sequence_files)
 
     def write_header(self, header):
         fmt = ('{0:' + str(self.max_name_length + 1) + 's}{1:10s}'
                 '{2:>10s}{3:>10s}{4:>10s}{5:>10s}')
-        print >> self.output, fmt.format(*header)
+        print(fmt.format(*header), file=self.output)
 
     def write_row(self, row):
         fmt = ('{name:' + str(self.max_name_length + 1) + 's}{alignment:10s}'
                 '{min_len:10d}{max_len:10d}{avg_len:10.2f}{num_seqs:10d}')
-        print >> self.output, fmt.format(**row._asdict())
-
+        print(fmt.format(**row._asdict()), file=self.output)
 
 _WRITERS = {'csv': CsvSeqInfoWriter, 'tab': TsvSeqInfoWriter, 'align':
         AlignedSeqInfoWriter}
@@ -93,7 +95,6 @@ _HEADERS = ('name', 'alignment', 'min_len', 'max_len', 'avg_len',
               'num_seqs')
 _SeqFileInfo = collections.namedtuple('SeqFileInfo', _HEADERS)
 
-
 def summarize_sequence_file(source_file, file_type=None):
     """
     Summarizes a sequence file, returning a tuple containing the name,
@@ -102,12 +103,12 @@ def summarize_sequence_file(source_file, file_type=None):
     """
     is_alignment = True
     avg_length = None
-    min_length = sys.maxint
+    min_length = sys.maxsize
     max_length = 0
     sequence_count = 0
 
     # Get an iterator and analyze the data.
-    with common.FileType('rb')(source_file) as fp:
+    with common.FileType('rt')(source_file) as fp:
         if not file_type:
             file_type = fileformat.from_handle(fp)
         for record in SeqIO.parse(fp, file_type):
@@ -138,7 +139,7 @@ def summarize_sequence_file(source_file, file_type=None):
     if sequence_count <= 1:
         is_alignment = False
 
-    return _SeqFileInfo(source_file, str(is_alignment).upper(), min_length,
+    return (source_file, str(is_alignment).upper(), min_length,
             max_length, avg_length, sequence_count)
 
 def action(arguments):
@@ -161,7 +162,18 @@ def action(arguments):
             output_format = 'tab'
 
     writer_cls = _WRITERS[output_format]
+
+    ssf = partial(summarize_sequence_file, file_type = arguments.input_format)
+
+    # if only one thread, do not use the multithreading so parent process
+    # can be terminated using ctrl+c
+    if arguments.threads > 1:
+        pool = multiprocessing.Pool(processes=arguments.threads)
+        summary = pool.imap(ssf, arguments.source_files)
+    else:
+        summary = (ssf(f) for f in arguments.source_files)
+
     with handle:
-        writer = writer_cls(arguments.source_files, handle,
-                arguments.input_format)
+        writer = writer_cls(arguments.source_files, summary, handle)
         writer.write()
+
