@@ -6,6 +6,7 @@ import logging
 
 from . import convert, common
 
+
 def build_parser(parser):
     """
     """
@@ -13,8 +14,7 @@ def build_parser(parser):
 
     parser.add_argument(
         'input_files', metavar="sequence_file", nargs='+',
-        type=common.FileType('r'),
-        help="Sequence file(s) to mogrify")
+        type=common.FileType('rt'), help="Sequence file(s) to mogrify")
 
     return parser
 
@@ -27,6 +27,8 @@ def action(arguments):
     for input_file in arguments.input_files:
         logging.info(input_file)
         # Generate a temporary file
-        with common.atomic_write(input_file.name,
-                file_factory=common.FileType('w')) as tf:
+        with common.atomic_write(
+                input_file.name, file_factory=common.FileType('wt')) as tf:
             convert.transform_file(input_file, tf, arguments)
+            if hasattr(input_file, 'close'):
+                input_file.close()
